package com.ruoyi.project.business.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.project.business.domain.TbCountryArea;
import com.ruoyi.project.business.mapper.TbCountryAreaMapper;
import com.ruoyi.project.business.service.ITbCountryAreaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 国家区域Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-04-16
 */
@Service
public class TbCountryAreaServiceImpl implements ITbCountryAreaService 
{
    @Autowired
    private TbCountryAreaMapper tbCountryAreaMapper;

    /**
     * 查询国家区域
     * 
     * @param countryId 国家区域主键
     * @return 国家区域
     */
    @Override
    public TbCountryArea selectTbCountryAreaByCountryId(Long countryId)
    {
        return tbCountryAreaMapper.selectTbCountryAreaByCountryId(countryId);
    }

    /**
     * 查询国家区域列表
     * 
     * @param tbCountryArea 国家区域
     * @return 国家区域
     */
    @Override
    public List<TbCountryArea> selectTbCountryAreaList(TbCountryArea tbCountryArea)
    {
        return tbCountryAreaMapper.selectTbCountryAreaList(tbCountryArea);
    }

    /**
     * 新增国家区域
     * 
     * @param tbCountryArea 国家区域
     * @return 结果
     */
    @Override
    public int insertTbCountryArea(TbCountryArea tbCountryArea)
    {
        tbCountryArea.setCreateTime(DateUtils.getNowDate());
        return tbCountryAreaMapper.insertTbCountryArea(tbCountryArea);
    }

    /**
     * 修改国家区域
     * 
     * @param tbCountryArea 国家区域
     * @return 结果
     */
    @Override
    public int updateTbCountryArea(TbCountryArea tbCountryArea)
    {
        tbCountryArea.setUpdateTime(DateUtils.getNowDate());
        return tbCountryAreaMapper.updateTbCountryArea(tbCountryArea);
    }

    /**
     * 批量删除国家区域
     * 
     * @param countryIds 需要删除的国家区域主键
     * @return 结果
     */
    @Override
    public int deleteTbCountryAreaByCountryIds(Long[] countryIds)
    {
        return tbCountryAreaMapper.deleteTbCountryAreaByCountryIds(countryIds);
    }

    /**
     * 删除国家区域信息
     * 
     * @param countryId 国家区域主键
     * @return 结果
     */
    @Override
    public int deleteTbCountryAreaByCountryId(Long countryId)
    {
        return tbCountryAreaMapper.deleteTbCountryAreaByCountryId(countryId);
    }
}
